#include "testlib.h"
#include <vector>
#include <algorithm>
#include <random>
#include <iostream>

typedef long long s64;

const int NR = 32;
const int N = 1e8;
const int T = 2e8;
const int MAX_PROG_LEN = 2e6;
const int P = 998244353;

inline bool chk(int x) {
    return -NR <= x && x < N;
}

int W;
int raw_s[NR + N];
int *s = raw_s + NR;

enum class OP {
    ADD,
    SUB,
    ADDM,
    SUBM,
    CPY,
    SET,
    CMP,
    JMP,
    ADV,
    MUL,
    DIV,
    COM
};

struct inst {
    OP op;
    int i, j, m, k;
};

int prog_len;
inst prog[MAX_PROG_LEN];

int max_cost = 0;

void run() {
    int cost = 0;
    int i, j, m, k;
    for (int p = 0; p < prog_len; p++) {
        i = prog[p].i, j = prog[p].j, m = prog[p].m, k = prog[p].k;
        switch (prog[p].op) {
            case OP::ADD:
                s[k] = (int)((unsigned)s[i] + (unsigned)s[j]);
                cost += k < 0 ? 1 : W;
                break;
            case OP::SUB:
                s[k] = (int)((unsigned)s[i] - (unsigned)s[j]);
                cost += k < 0 ? 1 : W;
                break;
            case OP::ADDM:
                if (s[m] <= 0) {
                    quitf(_wa, "ADDM: s[m] <= 0 at line %d", p + 1);
                }
                s[k] = ((s64)s[i] + (s64)s[j]) % s[m];
                if (s[k] < 0) {
                    s[k] += s[m];
                }
                cost += k < 0 ? 1 : W;
                break;
            case OP::SUBM:
                if (s[m] <= 0) {
                    quitf(_wa, "SUBM: s[m] <= 0 at line %d", p + 1);
                }
                s[k] = ((s64)s[i] - (s64)s[j]) % s[m];
                if (s[k] < 0) {
                    s[k] += s[m];
                }
                cost += k < 0 ? 1 : W;
                break;
            case OP::CPY:
                s[k] = s[i];
                cost += k < 0 ? 1 : W;
                break;
            case OP::SET:
                s[k] = m;
                cost += k < 0 ? 1 : W;
                break;
            case OP::CMP:
                s[k] = s[i] > s[j];
                cost += k < 0 ? 1 : W;
                break;
            case OP::JMP:
                if (s[i] == 0) {
                    p = m - 1;
                }
                cost++;
                break;
            case OP::ADV:
                if (!chk(s[i])) {
                    quitf(_wa, "ADV: out of range at line %d", p + 1);
                }
                s[k] = s[s[i]];
                cost += k < 0 ? 1 : W;
                break;
            case OP::MUL:
                if (s[m] <= 0) {
                    quitf(_wa, "MUL: s[m] <= 0 at line %d", p + 1);
                }
                s[k] = (s64)s[i] * s[j] % s[m];
                if (s[k] < 0) {
                    s[k] += s[m];
                }
                cost += k < 0 ? 1 : W;
                break;
            case OP::DIV:
                if (s[j] == 0) {
                    quitf(_wa, "DIV: division by zero");
                }
                s[k] = s[i] / s[j];
                cost += k < 0 ? 1 : W;
                break;
            case OP::COM:
                if (!chk(s[k])) {
                    quitf(_wa, "COM: out of range at line %d", p + 1);
                }
                s[s[k]] = s[i];
                cost += s[k] < 0 ? 1 : W;
                break;
        }
        if (cost > T) {
            quitf(_wa, "cost > 100000000");
        }
    }
    if (cost > max_cost) {
        max_cost = cost;
    }
}

void clear_s() {
    memset(raw_s, 0, sizeof(raw_s));
}

void test1(std::vector<int> v) {
    clear_s();
    copy(v.begin(), v.end(), s);

    sort(v.begin(), v.end());
    if (unique(v.begin(), v.end()) != v.end()) {
        return;
    }

    run();
    for (int i = 0; i < (int)v.size(); i++) {
        if (s[i] != v[i]) {
            quitf(_wa, "wrong answer");
        }
    }
}

void task1(const std::string &judge_type) {
    int n = 10000;
    W = 114;

    std::uniform_int_distribution<int> idx(0, n - 1);
    std::vector<int> v(n);

    std::mt19937_64 rng(23336633333333333ull);
    for (int i = 0; i < n; i++) {
        v[i] = rng();
    }
    test1(v);

    int pt = floor(40 * std::min(1.0, 10000000.0 / max_cost));
    if (pt == 40) {
        quitf(_ok, "cost = %d", max_cost);
    } else {
        quitp(pt, "cost = %d", max_cost);
    }
}

void test2(const std::vector<int> &mperm, const std::vector<int> &mval, const std::vector<int> &qperm) {
    int n = (int)mperm.size();

    std::vector<int> bit(n + 1), qy(n);

    auto upd = [&](int x, int y) {
        for (; x <= n; x += x & -x) {
            bit[x] = (int)((unsigned)bit[x] + (unsigned)y);
        }
    };
    auto qry = [&](int x) {
        int ans = 0;
        for (; x; x &= x - 1) {
            ans = (int)((unsigned)ans + (unsigned)bit[x]);
        }
        return ans;
    };

    clear_s();
    for (int i = 0; i < n; i++) {
        s[n + 3 * i] = mperm[i] + 1;
        s[n + 3 * i + 1] = mval[i];
        s[n + 3 * i + 2] = qperm[i] + 1;
        upd(mperm[i] + 1, mval[i]);
        qy[i] = qry(qperm[i] + 1);
    }

    run();

    for (int i = 0; i < n; ++i) {
        if (s[n + 3 * i + 2] != qy[i]) {
            quitf(_wa, "wrong answer");
        }
    }
}

void task2(const std::string &judge_type) {
    int n = 10000;
    W = 32;

    std::vector<int> mperm(n), mval(n), qperm(n);
    for (int i = 0; i < n; i++) {
        qperm[i] = i;
        mperm[i] = i;
    }

    std::mt19937_64 rng(23333333333366333ull);
    std::shuffle(mperm.begin(), mperm.end(), rng);
    for (int i = 0; i < n; i++) {
        mval[i] = rng();
    }
    std::shuffle(qperm.begin(), qperm.end(), rng);
    test2(mperm, mval, qperm);

    int pt = floor(30 * std::min(1.0, 2259485.0 / max_cost));
    if (pt == 30) {
        quitf(_ok, "cost = %d", max_cost);
    } else {
        quitp(pt, "cost = %d", max_cost);
    }
}

void test3(const std::vector<int> &a, const std::vector<int> &b) {
    int n = (int)a.size() - 1;

    clear_s();

    for (int i = 0; i <= n; i++) {
        s[i] = a[i];
        s[n + 1 + i] = b[i];
    }
    run();

    for (int i = 0; i <= 2 * n; ++i) {
        int qy = 0;
        for (int j = std::max(i - n, 0); j <= i && j <= n; j++) {
            qy += (s64)a[j] * b[i - j] % P;
            qy -= qy >= P ? P : 0;
        }
        if (qy != s[i]) {
            quitf(_wa, "wrong answer");
        }
    }
}

void task3(const std::string &judge_type) {
    int n = 2047;
    W = 16;
    std::uniform_int_distribution<int> dgt(0, P - 1);
    std::vector<int> a(n + 1), b(n + 1);

    std::mt19937_64 rng(233333333663333333ull);
    for (int i = 0; i <= n; i++) {
        a[i] = dgt(rng);
        b[i] = dgt(rng);
    }
    test3(a, b);

    if (max_cost > 6000000) {
        quitf(_wa, "cost = %d", max_cost);
    }

    int pt = floor(30 * std::min(1.0, 3000000.0 / max_cost));
    if (pt == 30) {
        quitf(_ok, "cost = %d", max_cost);
    } else {
        quitp(pt, "cost = %d", max_cost);
    }
}

int main(int argc, char * argv[]) {
    if (argc != 3) {
        fprintf(stderr, "usage: %s [output-source-code] [task-id]\n", argv[0]);
        return 1;
    }
	std::ifstream ouf(argv[1]);

    ::ouf.mode = _output;

    auto read = [&] {
        std::map<std::string, int> where;
        std::map<int, std::string> req;

        where["exit"] = -1;

        prog_len = 0;
        std::string label;
        std::string S;
        int p = 0;
        while (ouf >> S) {
            int i, j, m, k;

            p = prog_len;

            if (prog_len == MAX_PROG_LEN) {
                quitf(_wa, "the code is too long!");
            }

            if (S.length() >= 2 && S[S.length() - 1] == ':') {
                label = S.substr(0, S.length() - 1);
                if (!('a' <= label[0] && label[0] <= 'z') && !('A' <= label[0] && label[0] <= 'Z')) {
                    quitf(_wa, "invalid label at line %d", p + 1);
                }
                if (where.count(label)) {
                    quitf(_wa, "duplicate label at line %d", p + 1);
                }
                where[label] = p;

                if (!(ouf >> S)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
            }
            if (S == "ADD") {
                if (!(ouf >> i >> j >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::ADD, i, j, 0, k};
                if (!chk(i) || !chk(j) || !chk(k)) {
                    quitf(_wa, "ADD: out of range at line %d", p + 1);
                }
            } else if (S == "SUB") {
                if (!(ouf >> i >> j >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::SUB, i, j, 0, k};
                if (!chk(i) || !chk(j) || !chk(k)) {
                    quitf(_wa, "SUB: out of range at line %d", p + 1);
                }
            } else if (S == "ADDM") {
                if (!(ouf >> i >> j >> m >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::ADDM, i, j, m, k};
                if (!chk(i) || !chk(j) || !chk(m) || !chk(k)) {
                    quitf(_wa, "ADDM: out of range at line %d", p + 1);
                }
            } else if (S == "SUBM") {
                if (!(ouf >> i >> j >> m >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::SUBM, i, j, m, k};
                if (!chk(i) || !chk(j) || !chk(m) || !chk(k)) {
                    quitf(_wa, "SUBM: out of range at line %d", p + 1);
                }
            } else if (S == "CPY") {
                int i, k;
                if (!(ouf >> i >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::CPY, i, 0, 0, k};
                if (!chk(i) || !chk(k)) {
                    quitf(_wa, "CPY: out of range at line %d", p + 1);
                }
            } else if (S == "SET") {
                if (!(ouf >> m >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::SET, 0, 0, m, k};
                if (!chk(k)) {
                    quitf(_wa, "SET: out of range at line %d", p + 1);
                }
            } else if (S == "CMP") {
                int i, j, k;
                if (!(ouf >> i >> j >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::CMP, i, j, 0, k};
                if (!chk(i) || !chk(j) || !chk(k)) {
                    quitf(_wa, "CMP: out of range at line %d", p + 1);
                }
            } else if (S == "JMP") {
                if (!(ouf >> i >> S) || S.empty()) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                if ('0' <= S[0] && S[0] <= '9') {
                    m = atoi(S.c_str()) - 1;
                } else {
                    req[p] = S;
                    m = 0;
                }
                prog[prog_len++] = {OP::JMP, i, 0, m, k};
                if (!chk(i)) {
                    quitf(_wa, "JMP: out of range at line %d", p + 1);
                }
                if (m < 0) {
                    quitf(_wa, "JMP: l < 1 at line %d", p + 1);
                }
            } else if (S == "ADV") {
                if (!(ouf >> i >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::ADV, i, 0, 0, k};
                if (!chk(i) || !chk(k)) {
                    quitf(_wa, "ADV: out of range at line %d", p + 1);
                }
            } else if (S == "MUL") {
                if (!(ouf >> i >> j >> m >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::MUL, i, j, m, k};
                if (!chk(i) || !chk(j) || !chk(m) || !chk(k)) {
                    quitf(_wa, "MUL: out of range at line %d", p + 1);
                }
            } else if (S == "DIV") {
                if (!(ouf >> i >> j >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::DIV, i, j, 0, k};
                if (!chk(i) || !chk(j) || !chk(k)) {
                    quitf(_wa, "DIV: out of range at line %d", p + 1);
                }
            } else if (S == "COM") {
                if (!(ouf >> i >> k)) {
                    quitf(_wa, "invalid instruction #%d", p + 1);
                }
                prog[prog_len++] = {OP::COM, i, 0, 0, k};
                if (!chk(i) || !chk(k)) {
                    quitf(_wa, "COM: out of range at line %d", p + 1);
                }
            } else {
                quitf(_wa, "unknown instruction %s", S.c_str());
            }
        }

        where["exit"] = prog_len;
        for (auto &r : req) {
            if (!where.count(r.second)) {
                quitf(_wa, "JMP: invalid label at line %d", r.first + 1);
            }
            prog[r.first].m = where[r.second];
        }

        for (int i = 0; i < prog_len; i++) {
            if (prog[i].op == OP::JMP) {
                if (0 > prog[i].m || prog[i].m > prog_len) {
                    quitf(_wa, "JMP: invalid label at line %d", i + 1);
                }
            }
        }
    };

    read();

    int type = atoi(argv[2]);
    if (type < 0 || type > 3) {
        quitf(_fail, "task id should be in [0, 3]");
    }

    int n_in, n_out;
    switch (type) {
        case 0:
            std::cin >> W >> n_in >> n_out;
            for (int i = 0; i < n_in; i++) {
                std::cin >> s[i];
            }
            run();
            for (int i = 0; i < n_out; i++) {
                std::cout << s[i] << " ";
            }
            std::cout << std::endl;
            quitf(_ok, "hello xinchou");
        case 1:
            task1("sample");
            break;
        case 2:
            task2("sample");
            break;
        case 3:
            task3("sample");
            break;
    }
    return 0;
}
